package org.delta.winery.harvestwine.controllers;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;
import org.delta.dto.entities.grape.AddGrapeDTO;
import org.delta.dto.entities.grape.GrapeDTO;
import org.delta.dto.entities.harvest.AddHarvestDTO;
import org.delta.dto.entities.harvest.HarvestListEntityDTO;
import org.delta.dto.entities.harvest.UpdateHarvestDTO;
import org.delta.winery.harvestwine.facade.HarvestFacade;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.Optional;


@RestController
public class HarvestController {

    private final HarvestFacade harvestFacade;

    public HarvestController(HarvestFacade harvestFacade) {
        this.harvestFacade = harvestFacade;
    }

    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "getHarvests",
            summary = "Gets list of harvests",
            tags = {"HarvestController"},
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK", content = {
                            @Content(mediaType = "application/json", schema = @Schema(implementation = HarvestListEntityDTO.class))
                    })
            }
    )
    @GetMapping("/harvests")
    public ResponseEntity<Collection<HarvestListEntityDTO>> getHarvests(@RequestParam Optional<Long> grapeId, @RequestParam Optional<Integer> year) {
        return harvestFacade.getHarvestsWithParams(grapeId, year);
    }

    @PostMapping("/harvest")
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "addHarvest",
            summary = "Add harvest specified in the request body",
            tags = {"HarvestController"},
            responses = {
                    @ApiResponse(responseCode = "201", description = "Created", content = {
                            @Content(mediaType = "application/json", schema = @Schema(implementation = HarvestListEntityDTO.class)),
                    }),
                    @ApiResponse(responseCode = "400", description = "Harvest parameters are invalid.")
            }
    )
    @ResponseStatus(HttpStatus.CREATED)
    public ResponseEntity<HarvestListEntityDTO> addHarvest(@RequestBody AddHarvestDTO harvest) {
        return harvestFacade.addHarvest(harvest);
    }

    @PutMapping("/harvest")
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "editHarvest",
            summary = "Edit harvest specified in the request body",
            tags = {"HarvestController"},
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK"),
                    @ApiResponse(responseCode = "400", description = "Harvest with given id does not exist and cannot be edited.")
            }
    )
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<HarvestListEntityDTO> editHarvest(
            @NotNull @Valid @RequestBody @Schema(implementation = UpdateHarvestDTO.class) UpdateHarvestDTO harvest
    ) {
        return harvestFacade.updateHarvest(harvest);
    }

    @DeleteMapping("/harvests")
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "deleteHarvest",
            summary = "Delete harvest by its id",
            tags = {"HarvestController"},
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK"),
                    @ApiResponse(responseCode = "400", description = "Harvest with the given id does not exist.")
            }
    )
    public ResponseEntity<HarvestListEntityDTO> deleteHarvest(
            @NotNull @Parameter(name = "id", description = "Id of the harvest to be deleted", required = true, in = ParameterIn.QUERY) @Valid @RequestParam(value = "id", required = true) @PathVariable Long id
    ) {
        return harvestFacade.deleteHarvest(id);
    }


    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "getGrapes",
            summary = "Gets list of grapes",
            tags = {"HarvestController"},
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK", content = {
                            @Content(mediaType = "application/json", schema = @Schema(implementation = GrapeDTO.class))
                    })
            }
    )
    @GetMapping("/grapes")
    public ResponseEntity<Collection<GrapeDTO>> getGrapes() {
        return harvestFacade.getGrapes();
    }


    @PostMapping("/grapes")
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "addGrape",
            summary = "Add grape specified in the request body",
            tags = {"HarvestController"},
            responses = {
                    @ApiResponse(responseCode = "201", description = "Created", content = {
                            @Content(mediaType = "application/json", schema = @Schema(implementation = GrapeDTO.class)),
                    }),
                    @ApiResponse(responseCode = "400", description = "Grape parameters are invalid.")
            }
    )
    @ResponseStatus(HttpStatus.CREATED)
    public ResponseEntity<GrapeDTO> addGrape(@RequestBody AddGrapeDTO grape) {
        return harvestFacade.addGrape(grape);
    }

    @DeleteMapping("/grapes")
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "deleteGrape",
            summary = "Delete grape by its name",
            tags = {"HarvestController"},
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK"),
                    @ApiResponse(responseCode = "400", description = "Grape with the given name does not exist.")
            }
    )
    public ResponseEntity<GrapeDTO> deleteGrape(
            @NotNull @Parameter(name = "grapeName", description = "Name of the grape type to be deleted", required = true, in = ParameterIn.QUERY) @Valid @RequestParam(value = "grapeName", required = true) @PathVariable String grapeName
    ) {
        return harvestFacade.deleteGrape(grapeName);
    }

    @GetMapping("/harvests/canStore")
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "canBeStored",
            summary = "Check if given volume can be stored into tanks.",
            tags = {"HarvestController"},
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK"),
                    @ApiResponse(responseCode = "500", description = "Internal server error.")
            }
    )
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<Boolean> canBeStored(
            @RequestHeader HttpHeaders headers,
            @NotNull@Parameter(name = "volume", description = "Volume of wine juice that we want to store.", required = true, in = ParameterIn.QUERY) @RequestParam(value = "volume", required = true)@PathVariable BigDecimal volume
    ) {
        String bearerToken = headers.getFirst("Authorization").replace("Bearer ", "");
        return harvestFacade.canBeStored(volume, bearerToken);
    }

    @GetMapping("/harvests/surplus")
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "surplusValue",
            summary = "Returns the surplus value of the wine purchase.",
            tags = {"HarvestController"},
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK"),
                    @ApiResponse(responseCode = "500", description = "Internal server error.")
            }
    )
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<BigDecimal> surplusValue(
            @RequestHeader HttpHeaders headers,
            @NotNull @Parameter(name = "volumeInLiters", description = "Volume of wine juice that we want to buy.", required = true, in = ParameterIn.QUERY) @Valid @RequestParam(value = "volumeInLiters", required = true) @PathVariable BigDecimal volumeInLiters,
            @NotNull @Parameter(name = "purchasePricePerLiter", description = "Price of the wine juice.", required = true, in = ParameterIn.QUERY) @Valid @RequestParam(value = "purchasePricePerLiter", required = true) @PathVariable BigDecimal purchasePricePerLiter,
            @NotNull @Parameter(name = "bottlePrice", description = "Price of an empty bottle.", required = true, in = ParameterIn.QUERY) @Valid @RequestParam(value = "bottlePrice", required = true) @PathVariable BigDecimal bottlePrice,
            @NotNull @Parameter(name = "id", description = "Bottled wine id.", required = true, in = ParameterIn.QUERY) @Valid @RequestParam(value = "id", required = true) @PathVariable Long id
    ) {
        String bearerToken = headers.getFirst("Authorization").replace("Bearer ", "");
        return harvestFacade.surplusValue(volumeInLiters, purchasePricePerLiter, bottlePrice, id, bearerToken);
    }


}

