package org.delta.winery.harvestwine.service;

import org.delta.dto.entities.bottle.BottleDTO;
import org.delta.dto.entities.container.ContainerListEntityDTO;
import org.delta.dto.entities.grape.AddGrapeDTO;
import org.delta.dto.entities.harvest.AddHarvestDTO;
import org.delta.dto.entities.harvest.UpdateHarvestDTO;
import org.delta.winery.harvestwine.data.mapping.GrapeMapping;
import org.delta.winery.harvestwine.data.mapping.HarvestMapping;
import org.delta.winery.harvestwine.data.model.Grape;
import org.delta.winery.harvestwine.data.model.Harvest;
import org.delta.winery.harvestwine.data.repository.GrapeRepository;
import org.delta.winery.harvestwine.data.repository.HarvestRepository;
import org.delta.winery.harvestwine.helpers.UrlHelper;
import org.springframework.boot.web.client.RestTemplateBuilder;
import org.springframework.http.*;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.server.ResponseStatusException;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

@Service
public class HarvestManagementServiceImpl implements HarvestManagementService {
    private final HarvestRepository harvestRepository;
    private final GrapeRepository grapeRepository;
    private final RestTemplate restTemplate;

    public HarvestManagementServiceImpl(HarvestRepository harvestRepository, GrapeRepository grapeRepository, RestTemplateBuilder builder) {
        this.harvestRepository = harvestRepository;
        this.grapeRepository = grapeRepository;
        this.restTemplate = builder.build();
    }

    @Transactional(readOnly = true)
    public Collection<Harvest> getHarvestsByGrapeAndYear(Optional<Long> grapeId, Optional<Integer> year) {
        if (grapeId.isPresent() && year.isPresent()) {
            return harvestRepository.findByYearAndGrape(year.get(), grapeId.get());
        }
        if (grapeId.isPresent()) {
            return harvestRepository.findByGrape(grapeId.get());
        }
        if (year.isPresent()) {
            return harvestRepository.findByYear(year.get());
        }
        return harvestRepository.findAll();
    }

    @Transactional
    public Harvest addHarvest(AddHarvestDTO harvest) {
        Harvest h = HarvestMapping.getHarvestEntity(harvest, this.getGrape(harvest.getGrape().getGrapeName()));
        harvestRepository.save(h);
        return h;
    }

    @Override
    public Harvest updateHarvest(UpdateHarvestDTO harvest) {
        Harvest h = this.getHarvest(harvest.getId());
        h.setGrape(this.getGrape(harvest.getGrape().getGrapeName()));
        h.setYear(harvest.getYear());
        h.setVolume(harvest.getVolume());
        h.setPrice(harvest.getPrice());
        harvestRepository.save(h);
        return h;
    }

    @Override
    public Harvest getHarvest(Long id) {
        return harvestRepository.findById(id).orElseThrow(
                () -> new ResponseStatusException(HttpStatus.BAD_REQUEST, "Harvest with id " + id + " does not exist."));
    }

    @Override
    public Harvest deleteHarvest(Long id) {
        Harvest harvest = this.getHarvest(id);
        harvestRepository.delete(harvest);
        return harvest;
    }

    @Override
    public Collection<Grape> getGrapes() {
        return grapeRepository.findAll();
    }

    @Override
    public Grape addGrape(AddGrapeDTO grape) {
        Grape g = GrapeMapping.getGrapeEntity(grape);

        String errorMsg = "";
        if (g.getGrapeName() == null || g.getGrapeName() == "") {
            errorMsg = "Grape name cannot be empty or missing. ";
        }

        if (errorMsg != "") {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, "Cannot create harvest because: " + errorMsg);
        }

        grapeRepository.save(g);
        return g;
    }

    @Override
    public Grape getGrape(String grapeName) {
        return grapeRepository.findGrapeByName(grapeName).orElseThrow(
                () -> new ResponseStatusException(HttpStatus.BAD_REQUEST, "Grape with name " + grapeName + " does not exist."));
    }

    @Override
    public Grape deleteGrape(String grapeName) {
        Grape grape = this.getGrape(grapeName);
        grapeRepository.delete(grape);
        return grape;
    }

    @Override
    public boolean canBeStored(BigDecimal volume, String bearer) {
        String url = "http://" + UrlHelper.getLocalUrl("wine-production") + ":8082/stored/filter_containers?isEmpty=true";

        HttpHeaders headers = new HttpHeaders();
        headers.setBearerAuth(bearer);

        // Create HttpEntity object with headers and URI params
        HttpEntity<Object> entity = new HttpEntity<>(headers);

        ResponseEntity<ContainerListEntityDTO[]> response = restTemplate.exchange(
                url, HttpMethod.GET, entity, ContainerListEntityDTO[].class);

        ContainerListEntityDTO[] containers = response.getStatusCode() == HttpStatus.OK ? response.getBody() : null;

        BigDecimal capacitySum = BigDecimal.valueOf(0);
        assert containers != null;
        var capacities = Arrays.stream(containers).map(ContainerListEntityDTO::getCapacity)
                .toList();

        for (var capacity : capacities) {
            capacitySum = capacitySum.add(capacity);
        }

        return capacitySum.compareTo(volume) >= 0;
    }

    @Override
    public BigDecimal surplusValue(BigDecimal volumeInLiters, BigDecimal purchasePricePerLiter, BigDecimal emptyBottlePrice, Long id, String bearer) {
        String url = "http://" + UrlHelper.getLocalUrl("store-wine") + ":8083/store/id/" + id.toString();

        HttpHeaders headers = new HttpHeaders();
        headers.setBearerAuth(bearer);

        // Create HttpEntity object with headers and URI params
        HttpEntity<Object> entity = new HttpEntity<>(headers);

        ResponseEntity<BottleDTO> response = restTemplate.exchange(url, HttpMethod.GET, entity, BottleDTO.class);
        BottleDTO bottle = response.getStatusCode() == HttpStatus.OK ? response.getBody() : null;

        assert bottle != null;
        BigDecimal winePrice = bottle.getPrice().subtract(emptyBottlePrice);

        int bottlesToFill = volumeInLiters.divide(bottle.getVolume(), 2, RoundingMode.HALF_DOWN).intValue();

        BigDecimal finalWinePrice = winePrice.multiply(BigDecimal.valueOf(bottlesToFill));

        BigDecimal surplus = (purchasePricePerLiter.multiply(volumeInLiters)).subtract(finalWinePrice);

        String errorMsg = "";

        if (purchasePricePerLiter.compareTo(BigDecimal.valueOf(0)) <= 0) {
            errorMsg += "Invalid purchase price. ";
        }
        if (winePrice.compareTo(BigDecimal.valueOf(0)) <= 0) {
            errorMsg += "Emtpy bottle price equals full bottle price, there is no surplus value. Check the price of the bottle id. ";
        }
        if (emptyBottlePrice.compareTo(BigDecimal.valueOf(0)) <= 0) {
            errorMsg += "Invalid bottle price. ";
        }
        if (volumeInLiters.compareTo(BigDecimal.valueOf(0)) <= 0) {
            errorMsg += "Invalid volume of wine juice. ";
        }
        if (errorMsg != "") {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, "Surplus calculation unsuccessful because: " + errorMsg);
        }

        return surplus;
    }
}
