package org.delta.winery.harvestwine.tests;

import org.delta.dto.entities.grape.AddGrapeDTO;
import org.delta.dto.entities.harvest.AddHarvestDTO;
import org.delta.dto.entities.harvest.UpdateHarvestDTO;
import org.delta.winery.harvestwine.data.mapping.GrapeMapping;
import org.delta.winery.harvestwine.data.mapping.HarvestMapping;
import org.delta.winery.harvestwine.data.model.Grape;
import org.delta.winery.harvestwine.data.model.Harvest;
import org.delta.winery.harvestwine.data.repository.GrapeRepository;
import org.delta.winery.harvestwine.data.repository.HarvestRepository;
import org.delta.winery.harvestwine.service.HarvestManagementService;
import org.delta.winery.harvestwine.service.HarvestManagementServiceImpl;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.web.client.RestTemplateBuilder;
import org.springframework.web.server.ResponseStatusException;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;


@SpringBootTest
@AutoConfigureMockMvc
public class HarvestWineServiceTests {
    private HarvestManagementService harvestManagementService;
    private HarvestRepository harvestRepository;
    private GrapeRepository grapeRepository;
    @Autowired
    private RestTemplateBuilder builder;

    @BeforeEach
    public void setup() {
        harvestRepository = Mockito.mock(HarvestRepository.class);
        grapeRepository = Mockito.mock(GrapeRepository.class);

        harvestManagementService = new HarvestManagementServiceImpl(harvestRepository, grapeRepository, builder);
    }


    /*
    *   Service tests
    */
    @Test
    void getEmptyHarvestList() throws Exception {
        Assertions.assertTrue(harvestManagementService.getHarvestsByGrapeAndYear(Optional.empty(),Optional.empty()).size() == 0);
        Assertions.assertTrue(harvestManagementService.getGrapes().size() == 0);
    }

    @Test
    void addHarvestAndGrape() throws Exception {
        String grapeName = "test grape";
        AddGrapeDTO grapeDTO = new AddGrapeDTO();
        grapeDTO.setGrapeName(grapeName);

        when(grapeRepository.findGrapeByName(any(String.class))).thenReturn(Optional.of(GrapeMapping.getGrapeEntity(grapeDTO)));

        Grape grape = harvestManagementService.getGrape(grapeName);
        grape.setId(1L);

        AddHarvestDTO harvestDTO = new AddHarvestDTO();
        harvestDTO.setGrape(GrapeMapping.getGrapeDto(grape));
        harvestDTO.setPrice(BigDecimal.valueOf(50f));
        harvestDTO.setVolume(BigDecimal.valueOf(800));
        harvestDTO.setYear(2023);

        Harvest harvest = HarvestMapping.getHarvestEntity(harvestDTO, grape);
        harvest.setId(1L);

        List<Harvest> harvests = new ArrayList<>();
        harvests.add(harvest);

        when(harvestRepository.findAll()).thenReturn(harvests);

        Assertions.assertTrue(harvestManagementService.getHarvestsByGrapeAndYear(Optional.empty(),Optional.empty()).size() != 0);
    }

    @Test
    void deleteHarvest() throws Exception {
        String grapeName = "test grape";
        AddGrapeDTO grapeDTO = new AddGrapeDTO();
        grapeDTO.setGrapeName(grapeName);

        when(grapeRepository.findGrapeByName(any(String.class))).thenReturn(Optional.of(GrapeMapping.getGrapeEntity(grapeDTO)));

        Grape grape = harvestManagementService.getGrape(grapeName);
        grape.setId(1L);

        AddHarvestDTO harvestDTO = new AddHarvestDTO();
        harvestDTO.setGrape(GrapeMapping.getGrapeDto(grape));
        harvestDTO.setPrice(BigDecimal.valueOf(50f));
        harvestDTO.setVolume(BigDecimal.valueOf(800));
        harvestDTO.setYear(2023);

        Harvest harvest = HarvestMapping.getHarvestEntity(harvestDTO, grape);
        harvest.setId(1L);

        List<Harvest> harvests = new ArrayList<>();
        harvests.add(harvest);

        when(harvestRepository.findAll()).thenReturn(harvests);

        Assertions.assertTrue(harvestManagementService.getHarvestsByGrapeAndYear(Optional.empty(),Optional.empty()).size() != 0);
    }

    @Test
    void updateHarvestTest() throws Exception {
        when(grapeRepository.save(any(Grape.class))).thenReturn(getGrape());
        when(grapeRepository.findGrapeByName(any(String.class))).thenReturn(Optional.of(getGrape()));

        when(harvestRepository.findById(any(Long.class))).thenReturn(Optional.of(getHarvest()));

        Harvest harvest = harvestManagementService.getHarvest(1L);
        harvest.setPrice(BigDecimal.valueOf(850));
        when(harvestRepository.save(any(Harvest.class))).thenReturn(getHarvest());
        when(harvestRepository.findById(any(Long.class))).thenReturn(Optional.of(getHarvest()));

        UpdateHarvestDTO harvestDTO = new UpdateHarvestDTO();
        harvestDTO.setId(harvest.getId());
        harvestDTO.setGrape(GrapeMapping.getGrapeDto(getGrape()));
        harvestDTO.setPrice(harvest.getPrice());
        harvestDTO.setVolume(harvest.getVolume());
        harvestDTO.setYear(harvest.getYear());

        harvestManagementService.updateHarvest(harvestDTO);
        Harvest updatedHarvest = harvestManagementService.getHarvest(harvest.getId());

        Assertions.assertTrue(updatedHarvest != null && harvest.equals(updatedHarvest));
    }

    @Test
    void addGrapeTest() throws Exception {
        Grape grape = getGrape();
        when(grapeRepository.save(any(Grape.class))).thenReturn(grape);

        AddGrapeDTO grapeDTO = new AddGrapeDTO();
        grapeDTO.setGrapeName(grape.getGrapeName());

        Grape savedGrape = harvestManagementService.addGrape(grapeDTO);

        Assertions.assertEquals(grape, savedGrape);
    }

    @Test
    void deleteGrapeTest() throws Exception {
        Grape grape = getGrape();
        when(grapeRepository.findGrapeByName(any(String.class))).thenReturn(Optional.of(grape));
        harvestManagementService.deleteGrape(grape.getGrapeName());
        when(grapeRepository.findGrapeByName(any(String.class))).thenReturn(Optional.empty());
        Assertions.assertThrows(ResponseStatusException.class, () -> harvestManagementService.getGrape(grape.getGrapeName()));
    }

    private Grape getGrape() {
        Grape grape = new Grape();
        grape.setId(1L);
        grape.setGrapeName("Test grape");

        return grape;
    }

    private Harvest getHarvest() {
        Harvest harvest = new Harvest();
        harvest.setId(1L);
        harvest.setGrape(getGrape());
        harvest.setPrice(BigDecimal.valueOf(50f));
        harvest.setVolume(BigDecimal.valueOf(800));
        harvest.setYear(2023);
        return harvest;
    }
}
