package org.delta.winery.sellwine.controllers;

import io.swagger.annotations.ApiParam;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;
import org.delta.dto.entities.bottle.*;
import org.delta.dto.entities.order.*;
import org.delta.winery.sellwine.facade.SaleManagementFacade;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.Collection;
import java.util.Optional;

@RestController
@RequestMapping("/sale")
public class SaleController {

    private final SaleManagementFacade saleManagementFacade;

    public SaleController(SaleManagementFacade saleManagementFacade) {
        this.saleManagementFacade = saleManagementFacade;
    }

    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "getBottles",
            summary = "Gets list of bottles",
            tags = {"SaleController"},
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK", content = {
                            @Content(mediaType = "application/json", schema = @Schema(implementation = BottleListEntityDTO.class))
                    })
            }
    )
    @GetMapping("/bottle")
    public ResponseEntity<Collection<BottleListEntityDTO>> getBottles() {
        return saleManagementFacade.getBottles();
    }

    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "getBottle",
            summary = "Gets bottle by id",
            tags = { "SaleController" },
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK", content = {
                            @Content(mediaType = "application/json", schema = @Schema(implementation = BottleDTO.class))
                    }),
                    @ApiResponse(responseCode = "400", description = "Bottle not found")
            }
    )
    @GetMapping("/bottle/byId")
    public ResponseEntity<BottleDTO> getBottle(
            @Parameter(name = "id", description = "Bottle unique identifier", required = true, in = ParameterIn.QUERY) @Valid @RequestParam(value = "id", required = true) @ApiParam(required = true) Long id
    ) {
        return saleManagementFacade.getBottle(id);
    }

    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "getOrders",
            summary = "Gets list of orders",
            tags = {"SaleController"},
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK", content = {
                            @Content(mediaType = "application/json", schema = @Schema(implementation = OrderListEntityDTO.class))
                    })
            }
    )
    @GetMapping("/order")
    public ResponseEntity<Collection<OrderListEntityDTO>> getOrders() {
        return saleManagementFacade.getOrders();
    }

    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "getOrder",
            summary = "Gets order by id",
            tags = { "SaleController" },
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK", content = {
                            @Content(mediaType = "application/json", schema = @Schema(implementation = OrderDTO.class))
                    }),
                    @ApiResponse(responseCode = "400", description = "Order not found")
            }
    )
    @GetMapping("/order/byId")
    public ResponseEntity<OrderDTO> getOrder(
            @Parameter(name = "id", description = "Order unique identifier", required = true, in = ParameterIn.QUERY) @Valid @RequestParam(value = "id", required = true) @ApiParam(required = true) Long id
    ) {
        return saleManagementFacade.getOrder(id);
    }

    @PostMapping("/bottle")
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "addBottle",
            summary = "Add bottle specified in the request body",
            tags = { "SaleController" },
            responses = {
                    @ApiResponse(responseCode = "201", description = "Created", content = {
                            @Content(mediaType = "application/json", schema = @Schema(implementation = BottleDTO.class)),
                    }),
                    @ApiResponse(responseCode = "400", description = "Bottle parameters are invalid.")
            }
    )
    @ResponseStatus(HttpStatus.CREATED)
    public ResponseEntity<BottleDTO> addBottle(@NotNull @Valid @RequestBody @Schema(implementation = AddBottleDTO.class) AddBottleDTO bottle) {
        return saleManagementFacade.addBottle(bottle);
    }

    @PostMapping("/order")
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "addOrder",
            summary = "Add order specified in the request body",
            tags = { "SaleController" },
            responses = {
                    @ApiResponse(responseCode = "201", description = "Created", content = {
                            @Content(mediaType = "application/json", schema = @Schema(implementation = OrderDTO.class)),
                    }),
                    @ApiResponse(responseCode = "400", description = "Order parameters are invalid.")
            }
    )
    @ResponseStatus(HttpStatus.CREATED)
    public ResponseEntity<OrderDTO> addOrder(@NotNull @Valid @RequestBody @Schema(implementation = AddOrderDTO.class) AddOrderDTO order) {
        return saleManagementFacade.addOrder(order);
    }

    @PutMapping
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "editBottle",
            summary = "Edit bottle specified in the request body",
            tags = { "SaleController" },
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK"),
                    @ApiResponse(responseCode = "400", description = "Bottle with given id does not exist and cannot be edited.")
            }
    )
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<BottleDTO> editBottle(
            @NotNull @Valid @RequestBody @Schema(implementation = UpdateBottleDTO.class) UpdateBottleDTO bottle
    ) {
        return saleManagementFacade.updateBottle(bottle);
    }

    @PutMapping("/order/add_bottle")
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "addBottleToOrder",
            summary = "Add specified bottle to specified Order",
            tags = { "SaleController" },
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK"),
                    @ApiResponse(responseCode = "400", description = "Order with given id does not exist and cannot be edited.")
            }
    )
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<OrderDTO> addBottleToOrder(
            @NotNull @Valid @RequestBody @Schema(implementation = AddBottleToOrderDTO.class) AddBottleToOrderDTO order
    ) {
        return saleManagementFacade.addBottleToOrder(order);
    }

    @PutMapping("/order/remove_bottle")
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "removeBottleFromOrder",
            summary = "Remove specified Bottle from specified Order",
            tags = { "SaleController" },
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK"),
                    @ApiResponse(responseCode = "400", description = "Order with given id does not exist and cannot be edited.")
            }
    )
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<OrderDTO> removeBottleFromOrder(
            @NotNull @Valid @RequestBody @Schema(implementation = RemoveBottleFromOrderDTO.class) RemoveBottleFromOrderDTO order
    ) {
        return saleManagementFacade.removeBottleFromOrder(order);
    }

    @PostMapping("/order/complete_order")
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "completeOrder",
            summary = "Complete specified Order",
            tags = { "SaleController" },
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK"),
                    @ApiResponse(responseCode = "400", description = "Order with given id does not exist and cannot be edited.")
            }
    )
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<OrderDTO> completeOrder(
            @NotNull @Valid @RequestBody @Schema(implementation = OrderBaseDTO.class) OrderBaseDTO order
    ) {
        return saleManagementFacade.completeOrder(order);
    }

    @PostMapping("/order/leave_feedback")
    @Operation(
            security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "leaveFeedback",
            summary = "Leave feedback on a specified Order",
            tags = { "SaleController" },
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK"),
                    @ApiResponse(responseCode = "400", description = "Order with given id does not exist and cannot be edited.")
            }
    )
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<OrderDTO> leaveFeedback(
            @NotNull @Valid @RequestBody @Schema(implementation = UpdateOrderDTO.class) UpdateOrderDTO order
    ) {
        return saleManagementFacade.leaveFeedbackOnOrder(order);
    }

    @DeleteMapping("/bottle")
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "deleteBottle",
            summary = "Delete bottle by id",
            tags = { "SaleController" },
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK"),
                    @ApiResponse(responseCode = "400", description = "Bottle with the given id does not exist.")
            }
    )
    public ResponseEntity<BottleDTO> deleteBottle(
            @NotNull @Parameter(name = "id", description = "ID of the bottle to be deleted", required = true, in = ParameterIn.QUERY) @Valid @RequestParam(value = "id", required = true) @PathVariable Long id
    ) {
        return saleManagementFacade.deleteBottle(id);
    }

    @DeleteMapping("/order")
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "deleteOrder",
            summary = "Delete order by id",
            tags = { "SaleController" },
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK"),
                    @ApiResponse(responseCode = "400", description = "Order with the given id does not exist.")
            }
    )
    public ResponseEntity<OrderDTO> deleteOrder(
            @NotNull @Parameter(name = "id", description = "ID of the order to be deleted", required = true, in = ParameterIn.QUERY) @Valid @RequestParam(value = "id", required = true) @PathVariable Long id
    ) {
        return saleManagementFacade.deleteOrder(id);
    }

    @GetMapping("/search")
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "filterBottles",
            summary = "Get bottles by filter parameters. If parameter has default value or null, parameter is not used in filtering",
            tags = {"SaleController"},
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK", content = {
                            @Content(mediaType = "application/json", schema = @Schema(implementation = BottleListEntityDTO.class))
                    })
            }
    )
    public ResponseEntity<Collection<BottleListEntityDTO>> filterUsers(
            @Parameter(name = "Origin", description = "Origin of the bottle", allowEmptyValue = true) @RequestParam(value = "origin", required = false) Optional<String> origin,
            @Parameter(name = "Year", description = "Year when the bottle was produced", allowEmptyValue = true) @RequestParam(value = "year", required = false) Optional<Integer> year
    ) {
        return saleManagementFacade.getBottles(origin, year);
    }


}
