package org.delta.winery.sellwine.facade;

import org.delta.dto.entities.bottle.AddBottleDTO;
import org.delta.dto.entities.bottle.BottleDTO;
import org.delta.dto.entities.bottle.BottleListEntityDTO;
import org.delta.dto.entities.bottle.UpdateBottleDTO;
import org.delta.dto.entities.order.*;
import org.delta.winery.sellwine.data.mapping.BottleMapping;
import org.delta.winery.sellwine.data.mapping.OrderMapping;
import org.delta.winery.sellwine.data.model.Order;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collection;
import java.util.Optional;

public interface SaleManagementFacade {


    /**
     * Get all bottles stored in the database.
     * @return Collection of Bottles
     */
    ResponseEntity<Collection<BottleListEntityDTO>> getBottles();

    /**
     * Get bottles stored in the database filtered based on optional parameters
     * @param origin origin of the wine
     * @param year year the wine was produced
     * @return Collection of bottles
     */
    ResponseEntity<Collection<BottleListEntityDTO>> getBottles(Optional<String> origin, Optional<Integer> year);

    /**
     * Get all orders stored in the database.
     * @return Collection of Orders
     */
    ResponseEntity<Collection<OrderListEntityDTO>> getOrders();

    /**
     * Add Bottle to the database
     * @param bottle Parameters of the bottle to be stored in the database
     * @return Stored Bottle with entity ID
     */
    ResponseEntity<BottleDTO> addBottle(AddBottleDTO bottle);

    /**
     * Add Order to the database
     * @param order Parameters of the order to be stored in the database
     * @return Stored Order with entity ID
     */
    ResponseEntity<OrderDTO> addOrder(AddOrderDTO order);

    /**
     * Update bottle in the database
     * @param bottle parameters to be updated
     * @return the updated bottle
     */
    ResponseEntity<BottleDTO> updateBottle(UpdateBottleDTO bottle);

    /**
     * Adds bottle with specified ID to an order
     * @param order DTO containing bottle and order ID
     * @return Order with the added bottle
     */
    ResponseEntity<OrderDTO> addBottleToOrder(AddBottleToOrderDTO order);

    /**
     * Removes bottle with specified ID from order
     * @param order DTO containing bottle and order ID
     * @return Order state after update
     */
    ResponseEntity<OrderDTO> removeBottleFromOrder(RemoveBottleFromOrderDTO order);

    /**
     * set order as completed
     * @param order order to be completed
     * @return completed order state
     */
    ResponseEntity<OrderDTO> completeOrder(OrderBaseDTO order);


    /**
     * Leave feedback on specified order
     * @param order feedback to leave on a specified order
     * @return updated order
     */
    ResponseEntity<OrderDTO> leaveFeedbackOnOrder(UpdateOrderDTO order);

    /**
     * get specific bottle by its ID
     * @param id ID of the bottle to get
     * @return bottle with the specified ID
     */

    ResponseEntity<BottleDTO> getBottle(Long id);

    /**
     * get specific order by its ID
     * @param id ID of the order to get
     * @return order with the specified ID
     */

    ResponseEntity<OrderDTO> getOrder(Long id);

    /**
     * delete bottle by ID
     * @param id ID of the bottle we want to delete
     * @return deleted bottle
     */
    ResponseEntity<BottleDTO> deleteBottle(Long id);

    /**
     * delete order by ID
     * @param id ID of the order we want to delete
     * @return deleted order
     */
    ResponseEntity<OrderDTO> deleteOrder(Long id);
}
