package org.delta.winery.sellwine.service;

import org.delta.dto.entities.bottle.*;
import org.delta.dto.entities.order.*;
import org.delta.winery.sellwine.data.mapping.BottleMapping;
import org.delta.winery.sellwine.data.mapping.OrderMapping;
import org.delta.winery.sellwine.data.model.Bottle;
import org.delta.winery.sellwine.data.model.Order;
import org.delta.winery.sellwine.data.repository.BottleRepository;
import org.delta.winery.sellwine.data.repository.OrderRepository;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.server.ResponseStatusException;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.Optional;

@Service
public class SaleManagementServiceImpl implements SaleManagementService {

    private final BottleRepository bottleRepository;
    private final OrderRepository orderRepository;

    public SaleManagementServiceImpl(BottleRepository bottleRepository, OrderRepository orderRepository) {
        this.bottleRepository = bottleRepository;
        this.orderRepository = orderRepository;
    }

    @Transactional(readOnly = true)
    public Collection<Bottle> getBottles() {
        return bottleRepository.findAll();
    }

    @Transactional(readOnly = true)
    public Collection<Bottle> getBottles(Optional<String> origin, Optional<Integer> year) {
        return bottleRepository.filterBottles(origin, year);
    }

    @Transactional(readOnly = true)
    public Collection<Order> getOrders() {
        return orderRepository.findAll();
    }

    @Transactional
    public Bottle addBottle(AddBottleDTO bottleDTO) {
        Bottle bottle = BottleMapping.getBottleEntity(bottleDTO);

        String errorMsg = "";
        if (bottle.getPrice().compareTo(BigDecimal.valueOf(0)) <= 0) {
            errorMsg += "Price cannot be negative or 0. ";
        }
        if (bottle.getVolume().compareTo(BigDecimal.valueOf(0)) <= 0) {
            errorMsg += "Volume cannot be negative or 0. ";
        }
        if (bottle.getVolume().compareTo(BigDecimal.valueOf(0)) <= 0) {
            errorMsg += "Year cannot be negative or 0. ";
        }
        if (bottle.getOrigin() == null || bottle.getOrigin() == "") {
            errorMsg += "Origin cannot be empty or missing. ";
        }
        if (bottle.getAmount() <= 0) {
            errorMsg += "Amount cannot be negative or 0. ";
        }
        if (bottle.getName() == null || bottle.getOrigin() == "") {
            errorMsg += "Name cannot be empty or missing. ";
        }
        if (bottle.getType() == null) {
            errorMsg += "Wine type cannot be empty. ";
        }


        if (errorMsg != "") {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, "Cannot create bottle because: " + errorMsg);
        }

        bottleRepository.save(bottle);
        return bottle;
    }

    @Transactional
    public Order addOrder(AddOrderDTO orderDTO) {
        Order order = OrderMapping.getOrderEntity(orderDTO);
        order.setCompleted(false);

        orderRepository.save(order);
        return order;
    }

    @Transactional
    public Bottle updateBottle(UpdateBottleDTO bottleDTO) {
        Bottle bottle = getBottleOrThrow((long) bottleDTO.getId()); //TODO: change all ID's to long after end of Milestone
        bottle.setPrice(bottleDTO.getPrice());
        bottle.setAmount(bottleDTO.getAmount());

        bottleRepository.save(bottle);
        return bottle; //TODO: Add validation for updates
    }

    @Transactional
    public Order addBottleToOrder(AddBottleToOrderDTO orderDTO) {
        Order order = getOrderOrThrow(orderDTO.getId());
        Bottle bottle = getBottleOrThrow(orderDTO.getBottleId());
        order.addBottle(bottle);

        return order;
    }
    @Transactional
    public Order removeBottleFromOrder(RemoveBottleFromOrderDTO orderDTO) {
        Order order = getOrderOrThrow(orderDTO.getId());
        Bottle bottle = getBottleOrThrow(orderDTO.getBottleId());
        order.removeBottle(bottle);

        return order;
    }

    @Override
    public Order completeOrder(OrderBaseDTO orderDTO) {
        Order order = getOrderOrThrow(orderDTO.getId());
        order.setCompleted(true);
        orderRepository.save(order);

        return order;
    }

    @Override
    public Order leaveFeedbackOnOrder(UpdateOrderDTO orderDTO) {
        Order order = getOrderOrThrow(orderDTO.getId());
        order.setFeedback(orderDTO.getFeedback());
        if(!order.isCompleted()) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, "Cannot leave feedback on order that's not completed");
        }
        orderRepository.save(order);

        return order;
    }

    @Transactional(readOnly = true)
    public Bottle getBottle(Long id) {
        return getBottleOrThrow(id);
    }

    @Transactional(readOnly = true)
    public Order getOrder(Long id) {
        return getOrderOrThrow(id);

    }

    @Transactional
    public Bottle deleteBottle(Long id) {
        Bottle bottle = getBottleOrThrow(id);
        bottleRepository.delete(bottle);
        return bottle;
    }

    @Transactional
    public Order deleteOrder(Long id) {
        Order order = getOrderOrThrow(id);
        orderRepository.delete(order);
        return order;
    }

    private Bottle getBottleOrThrow(Long id) {
        return bottleRepository.findById(id)
                .orElseThrow(() -> new ResponseStatusException(HttpStatus.BAD_REQUEST, "Bottle with id " + id + " does not exist."));
    }

    private Order getOrderOrThrow(Long id) {
        return orderRepository.findById(id)
                .orElseThrow(() -> new ResponseStatusException(HttpStatus.BAD_REQUEST, "Order with id " + id + " does not exist."));
    }
}
