package org.delta.winery.sellwine.tests;

import org.delta.dto.entities.bottle.AddBottleDTO;
import org.delta.dto.entities.bottle.UpdateBottleDTO;
import org.delta.dto.entities.order.AddBottleToOrderDTO;
import org.delta.dto.entities.order.AddOrderDTO;
import org.delta.dto.entities.order.RemoveBottleFromOrderDTO;
import org.delta.dto.enums.WineType;
import org.delta.winery.sellwine.data.mapping.BottleMapping;
import org.delta.winery.sellwine.data.mapping.OrderMapping;
import org.delta.winery.sellwine.data.model.Bottle;
import org.delta.winery.sellwine.data.model.Order;
import org.delta.winery.sellwine.data.repository.BottleRepository;
import org.delta.winery.sellwine.data.repository.OrderRepository;
import org.delta.winery.sellwine.service.SaleManagementService;
import org.delta.winery.sellwine.service.SaleManagementServiceImpl;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.web.server.ResponseStatusException;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Optional;

import static org.junit.Assert.*;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.when;


@SpringBootTest
public class SellWineRepositoryTests {

    @Autowired
    private SaleManagementService salesManagementService;

    private BottleRepository bottleRepository;
    private OrderRepository orderRepository;

    @BeforeEach
    void setup() {
        bottleRepository = Mockito.mock(BottleRepository.class);
        orderRepository = Mockito.mock(OrderRepository.class);

        salesManagementService = new SaleManagementServiceImpl(bottleRepository, orderRepository);
    }

    @Test
    void getEmptyListTest() throws Exception {
        Assertions.assertTrue(salesManagementService.getBottles().size() == 0);
        Assertions.assertTrue(salesManagementService.getOrders().size() == 0);
    }


    @Test
    public void testGetOrders() {
        List<Order> orders = new ArrayList<>();
        orders.add(new Order());
        Mockito.when(orderRepository.findAll()).thenReturn(orders);

        Collection<Order> result = salesManagementService.getOrders();

        assertEquals(orders, result);
        Mockito.verify(orderRepository).findAll();
    }


    @Test
    public void testAddBottle() {
        AddBottleDTO bottle = new AddBottleDTO();
        bottle.setAmount(5);
        bottle.setOrigin("test");
        bottle.setPrice(BigDecimal.valueOf(1));
        bottle.setYear(5);
        bottle.setName("testwine");
        bottle.setWineType(WineType.RED);
        bottle.setVolume(BigDecimal.valueOf(0.7F));
        Mockito.when(bottleRepository.save(BottleMapping.getBottleEntity(bottle))).thenReturn(getBottle());

        Bottle result = salesManagementService.addBottle(bottle);

        assertEquals(5,result.getAmount());
        assertEquals("test",result.getOrigin());
        assertEquals(BigDecimal.valueOf(1),result.getPrice());
        assertEquals(5,result.getYear());
        assertEquals("testwine",result.getName());
        assertEquals(WineType.RED,result.getType());
        assertEquals(BigDecimal.valueOf(0.7F),result.getVolume());
    }

    @Test
    public void testDeleteBottle() {

        Bottle bottle = getBottle();
        bottle.setId(1L);
        when(bottleRepository.findById(any(Long.class))).thenReturn(Optional.of(bottle));
        salesManagementService.deleteBottle(bottle.getId());
        when(bottleRepository.findById(any(Long.class))).thenReturn(Optional.empty());
        assertThrows(ResponseStatusException.class, () -> salesManagementService.getBottle(bottle.getId()));
    }

    @Test
    public void testUpdateBottle() {
        UpdateBottleDTO bottleDTO = new UpdateBottleDTO();
        bottleDTO.setId(1);
        bottleDTO.setPrice(BigDecimal.valueOf(1));
        bottleDTO.setAmount(20);

        Bottle bottle = new Bottle();
        bottle.setId(1L);
        bottle.setPrice(BigDecimal.valueOf(2));
        bottle.setAmount(10);

        when(bottleRepository.findById((long) bottleDTO.getId())).thenReturn(Optional.of(bottle));

        Bottle result = salesManagementService.updateBottle(bottleDTO);

        assertEquals(result, bottle);
        assertEquals(bottleDTO.getPrice(), bottle.getPrice());
        assertEquals(bottleDTO.getAmount(), bottle.getAmount());

        Mockito.verify(bottleRepository).findById(bottleDTO.getId());
        Mockito.verify(bottleRepository).save(bottle);
    }

    @Test
    public void testAddOrder() {
        AddOrderDTO orderDTO = new AddOrderDTO();

        Order order = OrderMapping.getOrderEntity(orderDTO);

        when(orderRepository.save(order)).thenReturn(order);

        Order result = salesManagementService.addOrder(orderDTO);


        assertNotNull(result);
        assertEquals(order, result);
    }

    @Test
    public void testDeleteOrder() {
        Long orderId = 1L;
        Order order = new Order();
        when(orderRepository.findById(orderId)).thenReturn(Optional.of(order));

        Order result = salesManagementService.deleteOrder(orderId);

        assertEquals(result, order);
        Mockito.verify(orderRepository, times(1)).delete(order);
    }

    private Bottle getBottle() {
        Bottle bottle = new Bottle();
        bottle.setAmount(5);
        bottle.setOrigin("test");
        bottle.setPrice(BigDecimal.valueOf(1));
        bottle.setYear(5);
        bottle.setName("testwine");
        bottle.setType(WineType.RED);
        bottle.setVolume(BigDecimal.valueOf(0.7F));

        return bottle;
    }


    @Test
    public void testAddBottleToOrder() {
        Order order = new Order();
        Bottle bottle = new Bottle();
        bottle.setId(1L);
        bottle.setPrice(BigDecimal.valueOf(10));
        bottle.setAmount(5);


        when(orderRepository.findById(1L)).thenReturn(Optional.of(order));
        when(bottleRepository.findById(1L)).thenReturn(Optional.of(bottle));


        AddBottleToOrderDTO orderDTO = new AddBottleToOrderDTO();
        orderDTO.setId(1L);
        orderDTO.setBottleId(1L);


        Order result = salesManagementService.addBottleToOrder(orderDTO);
        assertEquals(result, order);


        assertTrue(order.getBottles().contains(bottle));
    }

    @Test
    public void testRemoveBottleFromOrder() {
        Order order = new Order();
        order.setId(1L);
        Bottle bottle = new Bottle();
        bottle.setId(1L);
        bottle.setPrice(BigDecimal.valueOf(10));
        bottle.setAmount(5);
        order.addBottle(bottle);

        when(orderRepository.findById(1L)).thenReturn(Optional.of(order));
        when(bottleRepository.findById(1L)).thenReturn(Optional.of(bottle));

        RemoveBottleFromOrderDTO orderDTO = new RemoveBottleFromOrderDTO();
        orderDTO.setId(1L);
        orderDTO.setBottleId(1L);

        Order result = salesManagementService.removeBottleFromOrder(orderDTO);
        assertEquals(result, order);


        assertFalse(order.getBottles().contains(bottle));
    }


}



