package org.delta.controller;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import org.delta.Facade.StoreFacade;
import org.delta.dto.entities.bottle.BottleDTO;
import org.delta.dto.entities.bottle.BottleListEntityDTO;
import org.delta.dto.entities.bottle.UpdateBottleDTO;
import org.delta.dto.entities.bottle.AddBottleDTO;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/store")
public class StoreWineController {
    private final StoreFacade storeFacade;

    public StoreWineController(StoreFacade storeFacadeImpl) {
        this.storeFacade = storeFacadeImpl;
    }

    @Operation(
            operationId = "getStoredBottles",
            summary = "Get all stored full bottles",
            tags = {"StoreWineController"},
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK", content = {
                            @Content(mediaType = "application/json", schema = @Schema(implementation = BottleListEntityDTO.class))
                    })
            }
    )
    @GetMapping("/bottles")
    public ResponseEntity<List<BottleListEntityDTO>> getStoredBottles() {
        return storeFacade.getStoredBottles();
    }

    @Operation(
            operationId = "getBottleById",
            summary = "Get bottle by id",
            tags = {"StoreWineController"},
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK", content = {
                            @Content(mediaType = "application/json", schema = @Schema(implementation = BottleDTO.class))
                    }),
                    @ApiResponse(responseCode = "400", description = "Invalid id of bottle")
            }
    )
    @GetMapping("/id/{id}")
    public ResponseEntity<BottleDTO> getBottleById(@PathVariable long id) {
        return storeFacade.getBottleById(id);
    }

    @Operation(
            operationId = "getBottleByName",
            summary = "Get bottle by name",
            tags = {"StoreWineController"},
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK", content = {
                            @Content(mediaType = "application/json", schema = @Schema(implementation = BottleDTO.class))
                    }),
                    @ApiResponse(responseCode = "400", description = "Invalid name of bottle")
            }
    )
    @GetMapping("/name/{name}")
    public ResponseEntity<BottleDTO> getBottleByName(@PathVariable String name) {
        return storeFacade.getBottleByName(name);
    }


    @Operation(
            operationId = "addBottle",
            summary = "Add new bottle to the store",
            tags = {"StoreWineController"},
            security = { @SecurityRequirement(name = "Bearer Authentication") },
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK", content = {
                            @Content(mediaType = "application/json", schema = @Schema(implementation = BottleDTO.class))
                    }),
                    @ApiResponse(responseCode = "400", description = "Invalid bottle parameters")
            }
    )
    @PostMapping("/bottle")
    public ResponseEntity<BottleDTO> addBottle(@RequestBody AddBottleDTO bottle) {
        return storeFacade.addBottle(bottle);

    }


    @Operation(
            operationId = "edit Bottle",
            summary = "Edit bottle specified in the request body",
            tags = {"StoreWineController"},
            security = { @SecurityRequirement(name = "Bearer Authentication") },
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK"),
                    @ApiResponse(responseCode = "400", description = "Bottle with given id does not exist and cannot be edited.")
            }
    )
    @ResponseStatus(HttpStatus.OK)
    @PutMapping
    public ResponseEntity<BottleDTO> editBottle(@RequestBody UpdateBottleDTO bottle) {
        return storeFacade.updateBottle(bottle);

    }

    @Operation(
            operationId = "deleteBottle",
            summary = "Delete bottle with given id",
            tags = { "StoreWineController" },
            security = { @SecurityRequirement(name = "Bearer Authentication") },
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK")
            }
    )

    @ResponseStatus(HttpStatus.OK)
    @DeleteMapping("/{id}")
    public ResponseEntity<BottleDTO> deleteBottle(@PathVariable long id) {
        return storeFacade.deleteBottle(id);

    }
}
