package org.delta.storewine.tests;

import org.delta.Data.Model.Bottle;
import org.delta.Data.Repository.StoreRepository;
import org.delta.dto.entities.bottle.AddBottleDTO;
import org.delta.dto.entities.bottle.UpdateBottleDTO;
import org.delta.dto.enums.WineType;
import org.delta.service.StoreWineService;
import org.delta.service.StoreWineServiceImpl;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.web.server.ResponseStatusException;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.List;
import java.util.Optional;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

@SpringBootTest
public class StoreWineTests {

    private StoreRepository storeRepository;
    @Autowired
    private StoreWineService storeWineService;

    @BeforeEach
    public void setup() {
        this.storeRepository = Mockito.mock(StoreRepository.class);
        this.storeWineService = new StoreWineServiceImpl(storeRepository);
    }


    @Test
    void getEmptyBottlesList() {
        assertTrue(storeWineService.getStoredBottles().size() == 0);
    }
    @Test
    void getStoredBottlesTest() {

        when(storeRepository.findAll()).thenReturn(List.of(getBottle()));
        Collection<Bottle> found = storeWineService.getStoredBottles();
        Bottle bottle = found.iterator().next();
        assertEquals(1,found.size());
        assertEquals(-1,bottle.getBottleID());
        assertEquals(5,bottle.getAmount());
        assertEquals("test",bottle.getOrigin());
        assertEquals(BigDecimal.valueOf(0),bottle.getPrice());
        assertEquals(5,bottle.getYear());
        assertEquals("testwine",bottle.getName());
        assertEquals(WineType.RED,bottle.getWineType());
        assertEquals(BigDecimal.valueOf(0.7F),bottle.getVolume());
    }

    @Test
    void addBottleTest() {
        AddBottleDTO addBottle = new AddBottleDTO();
        addBottle.setAmount(getBottle().getAmount());
        addBottle.setOrigin(getBottle().getOrigin());
        addBottle.setPrice(getBottle().getPrice());
        addBottle.setYear(getBottle().getYear());
        addBottle.setName(getBottle().getName());
        addBottle.setWineType(getBottle().getWineType());
        addBottle.setVolume(getBottle().getVolume());
        when(storeRepository.save(any())).thenReturn(getBottle());

        var bottle = storeWineService.addBottle(addBottle);

        assertEquals(5,bottle.getAmount());
        assertEquals("test",bottle.getOrigin());
        assertEquals(BigDecimal.valueOf(0),bottle.getPrice());
        assertEquals(5,bottle.getYear());
        assertEquals("testwine",bottle.getName());
        assertEquals(WineType.RED,bottle.getWineType());
        assertEquals(BigDecimal.valueOf(0.7F),bottle.getVolume());
    }

    @Test
    void updateBottleTest() {
        when(storeRepository.findByName(getBottle().getName())).thenReturn(Optional.of(getBottle()));

        UpdateBottleDTO updateBottle = new UpdateBottleDTO();
        updateBottle.setAmount(getBottle().getAmount());
        updateBottle.setOrigin(getBottle().getOrigin());
        updateBottle.setPrice(getBottle().getPrice());
        updateBottle.setYear(getBottle().getYear());
        updateBottle.setName(getBottle().getName());
        updateBottle.setWineType(getBottle().getWineType());
        updateBottle.setVolume(getBottle().getVolume());
        when(storeRepository.save(any())).thenReturn(getBottle());

        storeWineService.updateBottle(updateBottle);
        var bottle = storeWineService.getBottleByName(updateBottle.getName());

        assertEquals(5,bottle.getAmount());
        assertEquals("test",bottle.getOrigin());
        assertEquals(BigDecimal.valueOf(0),bottle.getPrice());
        assertEquals(5,bottle.getYear());
        assertEquals("testwine",bottle.getName());
        assertEquals(WineType.RED,bottle.getWineType());
        assertEquals(BigDecimal.valueOf(0.7F),bottle.getVolume());

    }

    @Test
    void getBottleByIdTest() {
        when(storeRepository.findById(getBottle().getBottleID())).thenReturn(Optional.of(getBottle()));
        var bottle = storeWineService.getBottleById(getBottle().getBottleID());
        assertEquals(5,bottle.getAmount());
        assertEquals("test",bottle.getOrigin());
        assertEquals(BigDecimal.valueOf(0),bottle.getPrice());
        assertEquals(5,bottle.getYear());
        assertEquals("testwine",bottle.getName());
        assertEquals(WineType.RED,bottle.getWineType());
        assertEquals(BigDecimal.valueOf(0.7F),bottle.getVolume());
    }

    @Test
    void getBottleByNameTest() {
        when(storeRepository.findByName(getBottle().getName())).thenReturn(Optional.of(getBottle()));
        var bottle = storeWineService.getBottleByName(getBottle().getName());
        assertEquals(5,bottle.getAmount());
        assertEquals("test",bottle.getOrigin());
        assertEquals(BigDecimal.valueOf(0),bottle.getPrice());
        assertEquals(5,bottle.getYear());
        assertEquals("testwine",bottle.getName());
        assertEquals(WineType.RED,bottle.getWineType());
        assertEquals(BigDecimal.valueOf(0.7F),bottle.getVolume());
    }

    @Test
    void deleteBottleTest() {
        Bottle bottle = getBottle();
        when(storeRepository.findById(any(Long.class))).thenReturn(Optional.of(bottle));
        storeWineService.deleteBottle(bottle.getBottleID());
        when(storeRepository.findById(any(Long.class))).thenReturn(Optional.empty());
        assertThrows(ResponseStatusException.class, () -> storeWineService.getBottleById(bottle.getBottleID()));

    }

    private Bottle getBottle(){
        Bottle bottle = new Bottle();
        bottle.setBottleID(-1);
        bottle.setAmount(5);
        bottle.setOrigin("test");
        bottle.setPrice(BigDecimal.valueOf(0));
        bottle.setYear(5);
        bottle.setName("testwine");
        bottle.setWineType(WineType.RED);
        bottle.setVolume(BigDecimal.valueOf(0.7F));
        return bottle;
    }

}
