package org.delta.winery.wineproduction.controllers;

import io.swagger.annotations.ApiParam;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;
import org.delta.dto.entities.container.AddContainerDTO;
import org.delta.dto.entities.container.ContainerDTO;
import org.delta.dto.entities.container.ContainerListEntityDTO;
import org.delta.dto.entities.container.UpdateContainerDTO;
import org.delta.dto.entities.wine.AddWineDTO;
import org.delta.dto.entities.wine.UpdateWineDTO;
import org.delta.dto.entities.wine.WineDTO;
import org.delta.dto.entities.wine.WineListEntityDTO;
import org.delta.dto.enums.WineState;
import org.delta.winery.wineproduction.data.model.WineContainer;
import org.delta.winery.wineproduction.facade.ProductionFacade;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.Optional;
import java.util.OptionalInt;

@RestController
@RequestMapping(path = "/stored")
public class ProductionController {

    private final ProductionFacade productionFacade;

    public ProductionController(ProductionFacade productionFacade) {
        this.productionFacade = productionFacade;
    }

    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "getWines",
            summary = "Gets list of wines",
            tags = { "ProductionController" },
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK", content = {
                            @Content(mediaType = "application/json", schema = @Schema(implementation = WineListEntityDTO.class))
                    })
            }
    )
    @GetMapping("/wines")
    public ResponseEntity<Collection<WineListEntityDTO>> getWines(
            @Parameter(name = "wineState", description = "Wines of certain state", in = ParameterIn.QUERY) @Valid @RequestParam(value = "wineState", required = false) @ApiParam(required = false) Optional<WineState> wineState
    ) {
        if (wineState.isEmpty()) {
            return productionFacade.getWines();
        }

        return productionFacade.getWineByState(wineState.get());
    }

    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "getWine",
            summary = "Gets wine by name",
            tags = { "ProductionController" },
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK", content = {
                            @Content(mediaType = "application/json", schema = @Schema(implementation = WineDTO.class))
                    }),
                    @ApiResponse(responseCode = "400", description = "Wine not found")
            }
    )
    @GetMapping("/wine_by_id")
    public ResponseEntity<WineDTO> getWine(
            @Parameter(name = "id", description = "Wine identifier", required = true, in = ParameterIn.QUERY)
            @Valid @RequestParam(value = "id", required = true)
            @ApiParam(required = true) Long id) {
        return productionFacade.getWine(id);
    }

    @PostMapping("/wines")
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "addWine",
            summary = "Add wine specified in the request body",
            tags = { "ProductionController" },
            responses = {
                    @ApiResponse(responseCode = "201", description = "Created", content = {
                            @Content(mediaType = "application/json", schema = @Schema(implementation = WineDTO.class)),
                    }),
                    @ApiResponse(responseCode = "400", description = "Wine parameters are invalid.")
            }
    )
    @ResponseStatus(HttpStatus.CREATED)
    public ResponseEntity<WineDTO> addWine(@NotNull @Valid @RequestBody @Schema(implementation = AddWineDTO.class) AddWineDTO wine, WineContainer container) {
        return productionFacade.addWine(wine, container);
    }

    @PutMapping("/wines")
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "editWine",
            summary = "Edit wine specified in the request body",
            tags = { "ProductionController" },
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK"),
                    @ApiResponse(responseCode = "400", description = "Wine with given id does not exist and cannot be edited.")
            }
    )
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<WineDTO> editWine(
            @NotNull @Valid @RequestBody @Schema(implementation = UpdateWineDTO.class) UpdateWineDTO wine
    ) {
      return productionFacade.updateWine(wine);
    }

    @DeleteMapping("/wines")
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "deleteWine",
            summary = "Delete wine by its id",
            tags = { "ProductionController" },
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK"),
                    @ApiResponse(responseCode = "400", description = "Wine with the given id does not exist.")
            }
    )
    public ResponseEntity<WineDTO> deleteWine(
            @NotNull @Parameter(name = "id", description = "id of the wine to be deleted", required = true, in = ParameterIn.QUERY)
            @Valid @RequestParam(value = "name", required = true) @PathVariable Long id
    ) {
      return productionFacade.deleteWine(id);
    }

    @GetMapping("/filter_wines")
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "filterWines",
            summary = "Get wines by filter parameters. If parameter has default value or null, parameter is not used in filtering",
            tags = { "ProductionController" },
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK", content = {
                            @Content(mediaType = "application/json", schema = @Schema(implementation = WineListEntityDTO.class)),
                    })
            }
    )
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<Collection<WineListEntityDTO>> filteredWine(
            @Parameter(name = "Name", description = "Name of the wine(if this parameter is set, result will always contain only one wine)", allowEmptyValue = true) @RequestParam(value = "name", required = false) Optional<String> name,
            @Parameter(name = "State", description = "State of the wine", allowEmptyValue = true) @RequestParam(value = "state", required = false) Optional<WineState> state,
            @Parameter(name = "Year", description = "Year of production", allowEmptyValue = true) @RequestParam(value = "year", required = false) OptionalInt year,
            @Parameter(name = "Grape Type", description = "Grape type of the wine", allowEmptyValue = true) @RequestParam(value = "grapeType", required = false) Optional<String> grapeType
    ) {
        return productionFacade.getWines(name, state, year, grapeType);
    }

    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "getContainers",
            summary = "Gets list of containers",
            tags = { "ProductionController" },
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK", content = {
                            @Content(mediaType = "application/json", schema = @Schema(implementation = WineListEntityDTO.class))
                    })
            }
    )
    @GetMapping("/containers")
    public ResponseEntity<Collection<ContainerListEntityDTO>> getContainers(){
        return productionFacade.getContainers();
    }

    @PostMapping("/containers")
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "addContainer",
            summary = "Add container specified in the request body",
            tags = { "ProductionController" },
            responses = {
                    @ApiResponse(responseCode = "201", description = "Created", content = {
                            @Content(mediaType = "application/json", schema = @Schema(implementation = ContainerDTO.class)),
                    }),
                    @ApiResponse(responseCode = "400", description = "Container parameters are invalid.")
            }
    )
    @ResponseStatus(HttpStatus.CREATED)
    public ResponseEntity<ContainerDTO> addContainer(@NotNull @Valid @RequestBody @Schema(implementation = AddContainerDTO.class) AddContainerDTO container) {
        return productionFacade.addContainer(container);
    }

    @PutMapping("/containers")
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "editContainer",
            summary = "Edit container specified in the request body",
            tags = { "ProductionController" },
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK"),
                    @ApiResponse(responseCode = "400", description = "Container with given id does not exist and cannot be edited.")
            }
    )
    public ResponseEntity<ContainerDTO> editContainer(
            @NotNull @Valid @RequestBody @Schema(implementation = UpdateContainerDTO.class) UpdateContainerDTO container
    ) {
        return productionFacade.updateContainer(container);
    }

    @DeleteMapping("/containers")
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "deleteContainer",
            summary = "Delete container by its id",
            tags = { "ProductionController" },
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK"),
                    @ApiResponse(responseCode = "400", description = "Container with the given id does not exist.")
            }
    )
    public ResponseEntity<ContainerDTO> deleteContainer(
            @NotNull @Parameter(name = "id", description = "id of the container to be deleted", required = true, in = ParameterIn.QUERY)
            @Valid @RequestParam(value = "id", required = true) @PathVariable Long id
    ) {
        return productionFacade.deleteContainer(id);
    }

    @GetMapping("/filter_containers")
    @Operation(
			security = { @SecurityRequirement(name = "Bearer Authentication") },
            operationId = "filterContainers",
            summary = "Get containers by filter parameters. If parameter has default value or null, parameter is not used in filtering",
            tags = { "ProductionController" },
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK", content = {
                            @Content(mediaType = "application/json", schema = @Schema(implementation = ContainerListEntityDTO.class)),
                    })
            }
    )
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<Collection<ContainerListEntityDTO>> filterContainer(
            @Parameter(name = "isEmpty", description = "For empty containers", allowEmptyValue = true) @RequestParam(value = "isEmpty", required = false) Optional<Boolean> isEmpty,
            @Parameter(name = "capacity", description = "Capacity of the container", allowEmptyValue = true) @RequestParam(value = "capacity", required = false) Optional<BigDecimal> capacity
    ) {
        return productionFacade.getContainers(isEmpty, capacity);
    }


}
