package org.delta.winery.wineproduction.facade;

import org.delta.dto.entities.container.AddContainerDTO;
import org.delta.dto.entities.container.ContainerDTO;
import org.delta.dto.entities.container.ContainerListEntityDTO;
import org.delta.dto.entities.container.UpdateContainerDTO;
import org.delta.dto.entities.wine.AddWineDTO;
import org.delta.dto.entities.wine.UpdateWineDTO;
import org.delta.dto.entities.wine.WineDTO;
import org.delta.dto.entities.wine.WineListEntityDTO;
import org.delta.dto.enums.WineState;
import org.delta.winery.wineproduction.data.model.Wine;
import org.delta.winery.wineproduction.data.model.WineContainer;
import org.springframework.http.ResponseEntity;
import org.springframework.web.server.ResponseStatusException;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.Optional;
import java.util.OptionalInt;

public interface ProductionFacade {

    /**
     * Get all stored wine.
     * @see Wine
     */
    public ResponseEntity<Collection<WineListEntityDTO>> getWines();


    /**
     * Get all stored wine based on parameters.
     * @see Wine
     */
    public ResponseEntity<Collection<WineListEntityDTO>> getWines
            (Optional<String> name, Optional<WineState> state, OptionalInt
                    year, Optional<String> grapeType);


    /**
     * Add wine to database.
     * @see Wine
     * @param wine that will be stored into database.
     * @param container that the wine shall be stored in.
     * @return Stored wine entity with ID.
     * @throws ResponseStatusException if wine cannot be created
     */
    public ResponseEntity<WineDTO> addWine(AddWineDTO wine, WineContainer container);
    /**
     * Get wines of given state.
     * @see Wine
     * @param state of wine to get.
     * @return Wines with given state.
     * @throws ResponseStatusException if wine is not found
     */
    public ResponseEntity<Collection<WineListEntityDTO>> getWineByState(WineState state);
    /**
     * Update wine in database.
     * @see UpdateWineDTO
     * @param wine Updated parameters of the bottle.
     * @return updated wine
     * @throws ResponseStatusException if bottle to be updated was not found.
     */
    public ResponseEntity<WineDTO> updateWine(UpdateWineDTO wine);

    /**
     * Find wine by id.
     * @see Wine
     * @param id of wine to get.
     * @return Wine with given id.
     * @throws ResponseStatusException if wine is not found
     */
    public ResponseEntity<WineDTO> getWine(Long id);

    /**
     * Delete wine by id.
     * @param id of wine to be deleted
     * @return deleted wine
     */
    public ResponseEntity<WineDTO> deleteWine(Long id);

    /**
     * Get all containers.
     * @see WineContainer
     */
    public ResponseEntity<Collection<ContainerListEntityDTO>> getContainers();

    /**
     * Get all containers based on parameters.
     * @see WineContainer
     */
    public ResponseEntity<Collection<ContainerListEntityDTO>> getContainers
            (Optional<Boolean> isEmpty, Optional<BigDecimal> capacity);
    /**
     * Add container to database.
     * @see WineContainer
     * @param container to add
     * @return Stored container entity with ID.
     * @throws ResponseStatusException if container cannot be created
     */
    public ResponseEntity<ContainerDTO> addContainer(AddContainerDTO container);
    /**
     * Update container in database.
     * @see UpdateContainerDTO
     * @param container Updated parameters of the bottle.
     * @return updated container
     * @throws ResponseStatusException if bottle to be updated was not found.
     */
    public ResponseEntity<ContainerDTO> updateContainer(UpdateContainerDTO container);
    /**
     * Find container by id.
     * @see WineContainer
     * @param id of container to get.
     * @return Container with given id.
     * @throws ResponseStatusException if container is not found
     */
    public ResponseEntity<ContainerDTO> getContainer(Long id);

    /**
     * Delete container by id.
     * @param id of container to be deleted
     * @return deleted container
     */
    public ResponseEntity<ContainerDTO> deleteContainer(Long id);

}
