package org.delta.winery.wineproduction.service;


import org.delta.dto.entities.container.AddContainerDTO;
import org.delta.dto.entities.container.UpdateContainerDTO;
import org.delta.dto.entities.wine.AddWineDTO;
import org.delta.dto.entities.wine.UpdateWineDTO;
import org.delta.dto.enums.WineState;
import org.delta.winery.wineproduction.data.mapping.ProductionMapping;
import org.delta.winery.wineproduction.data.model.Wine;
import org.delta.winery.wineproduction.data.model.WineContainer;
import org.delta.winery.wineproduction.data.repository.ContainerRepository;
import org.delta.winery.wineproduction.data.repository.ProductionRepository;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.server.ResponseStatusException;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.*;

@Service
public class WineProductionServiceImpl implements WineProductionService {

    private final ProductionRepository productionRepository;

    private final ContainerRepository containerRepository;

    public WineProductionServiceImpl(ProductionRepository productionRepository, ContainerRepository containerRepository) {
        this.productionRepository = productionRepository;
        this.containerRepository = containerRepository;
    }

    @Transactional(readOnly = true)
    public Collection<Wine> getWines() {
        return productionRepository.findAll();
    }

    @Override
    public Collection<Wine> getWines(Optional<String> name, Optional<WineState> state, OptionalInt year, Optional<String> grapeType) {
        return productionRepository.filterWine(name, state, year, grapeType);
    }

    @Override
    public Wine addWine(AddWineDTO wine, WineContainer container) {
        Wine w = ProductionMapping.getWineEntity(wine);
        LocalDate date = LocalDate.now();

        List<WineContainer> emptyContainers = containerRepository.findAll().stream().filter(c -> c.getEmpty()).toList();
        BigDecimal sum = BigDecimal.ZERO;
        List<WineContainer> selectedContainers = new ArrayList<>();
        boolean canStore = false;

        for(WineContainer emptyCont: emptyContainers) {
            selectedContainers.add(emptyCont);
            sum = sum.add(emptyCont.getCapacity());
            if (sum.compareTo(wine.getVolume()) >= 0) {
                canStore = true;
                break;
            }
        }

        String errorMsg = "";
        if (!canStore) {
            errorMsg = "Not enough empty containers ";
        }
        if (w.getName() == null || w.getName() == "") {
            errorMsg += "Wine name cannot be empty or missing. ";
        }
        if (w.getWineState() == null) {
            errorMsg += "Wine state cannot be empty. ";
        }
        if (w.getYear() != date.getYear()) {
            errorMsg += "Year does not match the current date. ";
        }
        if (w.getGrapeType() == null || w.getGrapeType()  == "") {
            errorMsg += "Grape type cannot be empty or missing. ";
        }
        if (errorMsg != "") {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, "Cannot add wine because: " + errorMsg);
        }

        for (WineContainer emptyCont: selectedContainers) {
            emptyCont.setEmpty(false);
            containerRepository.save(emptyCont);
        }

        return productionRepository.save(w);

    }

    @Transactional(readOnly = true)
    public List<Wine> getWineByState(WineState wineState) {
        return productionRepository.findWineByState(wineState);
    }

    @Transactional
    public Wine updateWine(UpdateWineDTO wine) {
        Wine w = getWineIdOrThrow(wine.getId());
        w.setName(wine.getName());
        w.setVolume(wine.getVolume());
        w.setWineState(wine.getWineState());
        w.setYear(wine.getYear());
        w.setGrapeType(wine.getGrapeType());

        productionRepository.save(w);
        return w;
    }

    @Override
    public Wine getWine(Long id) {
        return getWineIdOrThrow(id);
    }

    @Transactional
    public Wine deleteWine(Long id) {
        Wine wine = getWineIdOrThrow(id);
        productionRepository.delete(wine);
        return wine;
    }

    @Transactional(readOnly = true)
    public Collection<WineContainer> getWineContainers() {
        return containerRepository.findAll();
    }

    @Override
    public Collection<WineContainer> getWineContainers(Optional<Boolean> isEmpty, Optional<BigDecimal> capacity) {
        return containerRepository.filterContainers(isEmpty, capacity);
    }

    @Override
    public WineContainer addContainer(AddContainerDTO container) {
        WineContainer c = ProductionMapping.getContainerEntity(container);

        String errorMsg = "";
        if (c.getCapacity().compareTo(BigDecimal.valueOf(0)) <= 0) {
            errorMsg = "Container capacity must be greater than 0. ";
        }
        if (errorMsg != "") {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, "Cannot add container because: " + errorMsg);
        }

        return containerRepository.save(c);
    }

    @Override
    public WineContainer updateContainer(UpdateContainerDTO container) {
        WineContainer c = getContainerIdOrThrow(container.getId());
        c.setEmpty(container.getEmpty());
        c.setCapacity(container.getCapacity());

        containerRepository.save(c);
        return c;
    }

    @Override
    public WineContainer getContainer(Long id) {

        return getContainerIdOrThrow(id);
    }

    @Override
    public WineContainer deleteContainer(Long id) {
        WineContainer container = getContainerIdOrThrow(id);
        containerRepository.delete(container);
        return container;
    }

    private Wine getWineIdOrThrow(Long id) {
        return productionRepository.findById(id)
                .orElseThrow(() -> new ResponseStatusException(HttpStatus.BAD_REQUEST, "Wine with id " + id + " does not exist."));
    }

    private WineContainer getContainerIdOrThrow(Long id) {
        return containerRepository.findContainerById(id)
                .orElseThrow(() -> new ResponseStatusException(HttpStatus.BAD_REQUEST, "Container with id " + id + " does not exist."));
    }

}
